@@ -60,6 +60,12 @@ class BaseAudioConfig(Coqpit):
         trim_db (int):
             Silence threshold used for silence trimming. Defaults to 45.
 
+        do_rms_norm (bool, optional):
+            enable/disable RMS volume normalization when loading an audio file. Defaults to False.
+
+        db_level (int, optional):
+            dB level used for rms normalization. The range is -99 to 0. Defaults to None.
+
         power (float):
             Exponent used for expanding spectrogra levels before running Griffin Lim. It helps to reduce the
             artifacts in the synthesized voice. Defaults to 1.5.
@@ -116,6 +122,9 @@ class BaseAudioConfig(Coqpit):
     # silence trimming
     do_trim_silence: bool = True
     trim_db: int = 45
+    # rms volume normalization
+    do_rms_norm: bool = False
+    db_level: float = None
     # griffin-lim params
     power: float = 1.5
     griffin_lim_iters: int = 60
@@ -184,9 +193,12 @@ class BaseDatasetConfig(Coqpit):
             Name of the dataset meta file. Or a list of speakers to be ignored at training for multi-speaker datasets.
             Defaults to None.
 
-        unused_speakers (List):
+        ignored_speakers (List):
             List of speakers IDs that are not used at the training. Default None.
 
+        language (str):
+            Language code of the dataset. If defined, it overrides `phoneme_language`. Defaults to None.
+
         meta_file_val (str):
             Name of the dataset meta file that defines the instances used at validation.
 
@@ -198,7 +210,8 @@ class BaseDatasetConfig(Coqpit):
     name: str = ""
     path: str = ""
     meta_file_train: str = ""
-    ununsed_speakers: List[str] = None
+    ignored_speakers: List[str] = None
+    language: str = ""
     meta_file_val: str = ""
     meta_file_attn_mask: str = ""
 
@@ -335,6 +348,8 @@ class BaseTrainingConfig(Coqpit):
     num_loader_workers: int = 0
     num_eval_loader_workers: int = 0
     use_noise_augment: bool = False
+    use_language_weighted_sampler: bool = False
+
     # paths
     output_path: str = None
     # distributed
