@@ -12,7 +12,7 @@ from tqdm import tqdm
 ########################
 
 
-def tweb(root_path, meta_file):
+def tweb(root_path, meta_file, **kwargs):  # pylint: disable=unused-argument
     """Normalize TWEB dataset.
     https://www.kaggle.com/bryanpark/the-world-english-bible-speech-dataset
     """
@@ -28,7 +28,7 @@ def tweb(root_path, meta_file):
     return items
 
 
-def mozilla(root_path, meta_file):
+def mozilla(root_path, meta_file, **kwargs):  # pylint: disable=unused-argument
     """Normalizes Mozilla meta data files to TTS format"""
     txt_file = os.path.join(root_path, meta_file)
     items = []
@@ -43,7 +43,7 @@ def mozilla(root_path, meta_file):
     return items
 
 
-def mozilla_de(root_path, meta_file):
+def mozilla_de(root_path, meta_file, **kwargs):  # pylint: disable=unused-argument
     """Normalizes Mozilla meta data files to TTS format"""
     txt_file = os.path.join(root_path, meta_file)
     items = []
@@ -59,7 +59,7 @@ def mozilla_de(root_path, meta_file):
     return items
 
 
-def mailabs(root_path, meta_files=None):
+def mailabs(root_path, meta_files=None, ignored_speakers=None):
     """Normalizes M-AI-Labs meta data files to TTS format
 
     Args:
@@ -68,25 +68,34 @@ def mailabs(root_path, meta_files=None):
             recursively. Defaults to None
     """
     speaker_regex = re.compile("by_book/(male|female)/(?P<speaker_name>[^/]+)/")
-    if meta_files is None:
+    if not meta_files:
         csv_files = glob(root_path + "/**/metadata.csv", recursive=True)
     else:
         csv_files = meta_files
+
     # meta_files = [f.strip() for f in meta_files.split(",")]
     items = []
     for csv_file in csv_files:
-        txt_file = os.path.join(root_path, csv_file)
+        if os.path.isfile(csv_file):
+            txt_file = csv_file
+        else:
+            txt_file = os.path.join(root_path, csv_file)
+
         folder = os.path.dirname(txt_file)
         # determine speaker based on folder structure...
         speaker_name_match = speaker_regex.search(txt_file)
         if speaker_name_match is None:
             continue
         speaker_name = speaker_name_match.group("speaker_name")
+        # ignore speakers
+        if isinstance(ignored_speakers, list):
+            if speaker_name in ignored_speakers:
+                continue
         print(" | > {}".format(csv_file))
         with open(txt_file, "r", encoding="utf-8") as ttf:
             for line in ttf:
                 cols = line.split("|")
-                if meta_files is None:
+                if not meta_files:
                     wav_file = os.path.join(folder, "wavs", cols[0] + ".wav")
                 else:
                     wav_file = os.path.join(root_path, folder.replace("metadata.csv", ""), "wavs", cols[0] + ".wav")
@@ -94,11 +103,12 @@ def mailabs(root_path, meta_files=None):
                     text = cols[1].strip()
                     items.append([text, wav_file, speaker_name])
                 else:
-                    raise RuntimeError("> File %s does not exist!" % (wav_file))
+                    # M-AI-Labs have some missing samples, so just print the warning
+                    print("> File %s does not exist!" % (wav_file))
     return items
 
 
-def ljspeech(root_path, meta_file):
+def ljspeech(root_path, meta_file, **kwargs):  # pylint: disable=unused-argument
     """Normalizes the LJSpeech meta data file to TTS format
     https://keithito.com/LJ-Speech-Dataset/"""
     txt_file = os.path.join(root_path, meta_file)
@@ -113,7 +123,7 @@ def ljspeech(root_path, meta_file):
     return items
 
 
-def ljspeech_test(root_path, meta_file):
+def ljspeech_test(root_path, meta_file, **kwargs):  # pylint: disable=unused-argument
     """Normalizes the LJSpeech meta data file for TTS testing
     https://keithito.com/LJ-Speech-Dataset/"""
     txt_file = os.path.join(root_path, meta_file)
@@ -127,7 +137,7 @@ def ljspeech_test(root_path, meta_file):
     return items
 
 
-def sam_accenture(root_path, meta_file):
+def sam_accenture(root_path, meta_file, **kwargs):  # pylint: disable=unused-argument
     """Normalizes the sam-accenture meta data file to TTS format
     https://github.com/Sam-Accenture-Non-Binary-Voice/non-binary-voice-files"""
     xml_file = os.path.join(root_path, "voice_over_recordings", meta_file)
@@ -144,12 +154,12 @@ def sam_accenture(root_path, meta_file):
     return items
 
 
-def ruslan(root_path, meta_file):
+def ruslan(root_path, meta_file, **kwargs):  # pylint: disable=unused-argument
     """Normalizes the RUSLAN meta data file to TTS format
     https://ruslan-corpus.github.io/"""
     txt_file = os.path.join(root_path, meta_file)
     items = []
-    speaker_name = "ljspeech"
+    speaker_name = "ruslan"
     with open(txt_file, "r", encoding="utf-8") as ttf:
         for line in ttf:
             cols = line.split("|")
@@ -159,11 +169,11 @@ def ruslan(root_path, meta_file):
     return items
 
 
-def css10(root_path, meta_file):
+def css10(root_path, meta_file, **kwargs):  # pylint: disable=unused-argument
     """Normalizes the CSS10 dataset file to TTS format"""
     txt_file = os.path.join(root_path, meta_file)
     items = []
-    speaker_name = "ljspeech"
+    speaker_name = "css10"
     with open(txt_file, "r", encoding="utf-8") as ttf:
         for line in ttf:
             cols = line.split("|")
@@ -173,7 +183,7 @@ def css10(root_path, meta_file):
     return items
 
 
-def nancy(root_path, meta_file):
+def nancy(root_path, meta_file, **kwargs):  # pylint: disable=unused-argument
     """Normalizes the Nancy meta data file to TTS format"""
     txt_file = os.path.join(root_path, meta_file)
     items = []
@@ -187,7 +197,7 @@ def nancy(root_path, meta_file):
     return items
 
 
-def common_voice(root_path, meta_file):
+def common_voice(root_path, meta_file, ignored_speakers=None):
     """Normalize the common voice meta data file to TTS format."""
     txt_file = os.path.join(root_path, meta_file)
     items = []
@@ -198,15 +208,19 @@ def common_voice(root_path, meta_file):
             cols = line.split("\t")
             text = cols[2]
             speaker_name = cols[0]
+            # ignore speakers
+            if isinstance(ignored_speakers, list):
+                if speaker_name in ignored_speakers:
+                    continue
             wav_file = os.path.join(root_path, "clips", cols[1].replace(".mp3", ".wav"))
             items.append([text, wav_file, "MCV_" + speaker_name])
     return items
 
 
-def libri_tts(root_path, meta_files=None):
+def libri_tts(root_path, meta_files=None, ignored_speakers=None):
     """https://ai.google/tools/datasets/libri-tts/"""
     items = []
-    if meta_files is None:
+    if not meta_files:
         meta_files = glob(f"{root_path}/**/*trans.tsv", recursive=True)
     else:
         if isinstance(meta_files, str):
@@ -222,13 +236,17 @@ def libri_tts(root_path, meta_files=None):
                 _root_path = os.path.join(root_path, f"{speaker_name}/{chapter_id}")
                 wav_file = os.path.join(_root_path, file_name + ".wav")
                 text = cols[2]
+                # ignore speakers
+                if isinstance(ignored_speakers, list):
+                    if speaker_name in ignored_speakers:
+                        continue
                 items.append([text, wav_file, "LTTS_" + speaker_name])
     for item in items:
         assert os.path.exists(item[1]), f" [!] wav files don't exist - {item[1]}"
     return items
 
 
-def custom_turkish(root_path, meta_file):
+def custom_turkish(root_path, meta_file, **kwargs):  # pylint: disable=unused-argument
     txt_file = os.path.join(root_path, meta_file)
     items = []
     speaker_name = "turkish-female"
@@ -247,7 +265,7 @@ def custom_turkish(root_path, meta_file):
 
 
 # ToDo: add the dataset link when the dataset is released publicly
-def brspeech(root_path, meta_file):
+def brspeech(root_path, meta_file, ignored_speakers=None):
     """BRSpeech 3.0 beta"""
     txt_file = os.path.join(root_path, meta_file)
     items = []
@@ -258,21 +276,25 @@ def brspeech(root_path, meta_file):
             cols = line.split("|")
             wav_file = os.path.join(root_path, cols[0])
             text = cols[2]
-            speaker_name = cols[3]
-            items.append([text, wav_file, speaker_name])
+            speaker_id = cols[3]
+            # ignore speakers
+            if isinstance(ignored_speakers, list):
+                if speaker_id in ignored_speakers:
+                    continue
+            items.append([text, wav_file, speaker_id])
     return items
 
 
-def vctk(root_path, meta_files=None, wavs_path="wav48"):
+def vctk(root_path, meta_files=None, wavs_path="wav48", ignored_speakers=None):
     """homepages.inf.ed.ac.uk/jyamagis/release/VCTK-Corpus.tar.gz"""
-    test_speakers = meta_files
     items = []
     meta_files = glob(f"{os.path.join(root_path,'txt')}/**/*.txt", recursive=True)
     for meta_file in meta_files:
         _, speaker_id, txt_file = os.path.relpath(meta_file, root_path).split(os.sep)
         file_id = txt_file.split(".")[0]
-        if isinstance(test_speakers, list):  # if is list ignore this speakers ids
-            if speaker_id in test_speakers:
+        # ignore speakers
+        if isinstance(ignored_speakers, list):
+            if speaker_id in ignored_speakers:
                 continue
         with open(meta_file, "r", encoding="utf-8") as file_text:
             text = file_text.readlines()[0]
@@ -282,15 +304,16 @@ def vctk(root_path, meta_files=None, wavs_path="wav48"):
     return items
 
 
-def vctk_slim(root_path, meta_files=None, wavs_path="wav48"):
+def vctk_slim(root_path, meta_files=None, wavs_path="wav48", ignored_speakers=None):  # pylint: disable=unused-argument
     """homepages.inf.ed.ac.uk/jyamagis/release/VCTK-Corpus.tar.gz"""
     items = []
     txt_files = glob(f"{os.path.join(root_path,'txt')}/**/*.txt", recursive=True)
     for text_file in txt_files:
         _, speaker_id, txt_file = os.path.relpath(text_file, root_path).split(os.sep)
         file_id = txt_file.split(".")[0]
-        if isinstance(meta_files, list):  # if is list ignore this speakers ids
-            if speaker_id in meta_files:
+        # ignore speakers
+        if isinstance(ignored_speakers, list):
+            if speaker_id in ignored_speakers:
                 continue
         wav_file = os.path.join(root_path, wavs_path, speaker_id, file_id + ".wav")
         items.append([None, wav_file, "VCTK_" + speaker_id])
@@ -298,7 +321,7 @@ def vctk_slim(root_path, meta_files=None, wavs_path="wav48"):
     return items
 
 
-def mls(root_path, meta_files=None):
+def mls(root_path, meta_files=None, ignored_speakers=None):
     """http://www.openslr.org/94/"""
     items = []
     with open(os.path.join(root_path, meta_files), "r", encoding="utf-8") as meta:
@@ -307,19 +330,23 @@ def mls(root_path, meta_files=None):
             text = text[:-1]
             speaker, book, *_ = file.split("_")
             wav_file = os.path.join(root_path, os.path.dirname(meta_files), "audio", speaker, book, file + ".wav")
+            # ignore speakers
+            if isinstance(ignored_speakers, list):
+                if speaker in ignored_speakers:
+                    continue
             items.append([text, wav_file, "MLS_" + speaker])
     return items
 
 
 # ======================================== VOX CELEB ===========================================
-def voxceleb2(root_path, meta_file=None):
+def voxceleb2(root_path, meta_file=None, **kwargs):  # pylint: disable=unused-argument
     """
     :param meta_file   Used only for consistency with load_tts_samples api
     """
     return _voxcel_x(root_path, meta_file, voxcel_idx="2")
 
 
-def voxceleb1(root_path, meta_file=None):
+def voxceleb1(root_path, meta_file=None, **kwargs):  # pylint: disable=unused-argument
     """
     :param meta_file   Used only for consistency with load_tts_samples api
     """
@@ -361,7 +388,7 @@ def _voxcel_x(root_path, meta_file, voxcel_idx):
         return [x.strip().split("|") for x in f.readlines()]
 
 
-def baker(root_path: str, meta_file: str) -> List[List[str]]:
+def baker(root_path: str, meta_file: str, **kwargs) -> List[List[str]]:  # pylint: disable=unused-argument
     """Normalizes the Baker meta data file to TTS format
 
     Args:
@@ -381,7 +408,7 @@ def baker(root_path: str, meta_file: str) -> List[List[str]]:
     return items
 
 
-def kokoro(root_path, meta_file):
+def kokoro(root_path, meta_file, **kwargs):  # pylint: disable=unused-argument
     """Japanese single-speaker dataset from https://github.com/kaiidams/Kokoro-Speech-Dataset"""
     txt_file = os.path.join(root_path, meta_file)
     items = []
